#!/usr/bin/env python
#
#     outputs.py: functions for outputting result data
#     Copyright (C) University of Manchester 2018-2021 Mudassar Iqbal, Peter Briggs
#

#######################################################################
# Constants
#######################################################################

# Default axis labels
CLUSTERS_AXIS_LABEL = "RNA-seq Clusters"
PEAKSETS_AXIS_LABEL = "Peak Sets and Intervals"
# Sets font size for heatmap annotation
ANNOTATION_FONT_SIZE = 16
# Sets font size for peakset names associated with distances
# on y-axis
PEAKSET_LABEL_FONT_SIZE = 16
# Sets font size for tick labels on all x- and y-axes
X_TICK_LABEL_FONT_SIZE = 16
Y_TICK_LABEL_FONT_SIZE = 16
# Sets font size for the x- and y-axes labels
AXIS_LABEL_FONT_SIZE = 24

#######################################################################
# Imports
#######################################################################

from builtins import str
import io
import os
import numpy as np
# Deal with matplotlib backend
# See https://stackoverflow.com/a/50089385/579925
import matplotlib
if os.environ.get('DISPLAY','') == '':
   print('No display found: using non-interactive Agg backend')
   matplotlib.use('Agg')
import matplotlib.pyplot as plt
import seaborn as sns
import xlsxwriter

from os.path import basename
from os.path import splitext

#######################################################################
# Functions
#######################################################################

def make_heatmap(heatmap_file,peaks,clusters,distances,pvalues,counts,
                 tads_pvalues=None,tads_counts=None,
                 clusters_axis_label=None,
                 peaksets_axis_label=None,
                 heatmap_cmap=None,heatmap_format=None):
    """
    Generate a heatmap from enrichment data

    Arguments:
      heatmap_file (str): name/path for output heatmap PNG
      peaks (list): BED files containing the ChIP-seq peaks
      clusters (list): list of distances to calculate enrichments at
      distances (list): cluster files
      pvalues (numpy.array): Numpy array with pvalues from enrichment
        calculation
      counts (numpy.array): Numpy array with gene counts from enrichment
        calculation
      tads_pvalues (numpy.array): Numpy array with TADs pvalues from
        enrichment calculation (None if TADs not included)
      tads_counts (numpy.array): Numpy array with TADs gene counts
        from enrichment calculation (None if TADs not included)
      clusters_axis_label (str): custom label for the x-axis
      peaksets_axis_label (str): custom label for the y-axis
      heatmap_cmap (cmap): optional, colormap to use when plotting
        the heatmaps
      heatmap_format (str): optional, image format for output heatmaps
    """
    # Defaults for axis labels
    if clusters_axis_label is None:
       clusters_axis_label = CLUSTERS_AXIS_LABEL
    if peaksets_axis_label is None:
       peaksets_axis_label = PEAKSETS_AXIS_LABEL

    # Convenience variables
    n_peaks = len(peaks)
    n_clusters = len(clusters)
    n_distances = len(distances)
    include_tads = (tads_pvalues is not None) and \
                   (tads_counts is not None)

    # Make '2d' versions of enrichment data for plotting heatmap
    pvalues_2d = np.zeros([n_peaks*n_distances,n_clusters])
    counts_2d = np.zeros([n_peaks*n_distances,n_clusters])
    for i,peaks_file in enumerate(peaks):
        for j,distance in enumerate(distances):
            x = i*n_distances + j
            pvalues_2d[x,:] = pvalues[i,j,:]
            counts_2d[x,:] = counts[i,j,:]

    # Min/max pvalues for colorbar
    min_pvalue = np.amin(-np.log10(pvalues_2d))
    max_pvalue = np.amax(-np.log10(pvalues_2d))
    if include_tads:
        min_pvalue = min(min_pvalue,np.amin(-np.log10(tads_pvalues)))
        max_pvalue = max(max_pvalue,np.amax(-np.log10(tads_pvalues)))

    # Xlabel (cluster names)
    xlbls = [os.path.splitext(os.path.basename(x))[0]
             for x in clusters]

    # Ylabel (interval distances repeated for each peak set)
    ylbls = [d for d in distances] * n_peaks

    # Set up grid and axes for plotting the heatmaps and
    # associated colorbars
    # (see https://stackoverflow.com/a/45645152)
    if include_tads:
        # 2 x 2 grid to include TADS
        fig,axes = plt.subplots(2,2,
                                gridspec_kw={
                                    'height_ratios': [5,1],
                                    'width_ratios': [30,1],
                                    'hspace': 0.1,
                                    'wspace': 0.05,
                                })
        # Axes for main heatmap and colorbar
        ax = axes[0][0]
        cbar_ax = axes[0][1]
        # Axes for TADs heatmap and colorbar
        tads_ax = axes[1][0]
        tads_cbar_ax = axes[1][1]
    else:
        # 1 x 2 grid (no TADS)
        fig,axes = plt.subplots(1,2,
                                gridspec_kw={
                                    'height_ratios': [5],
                                    'width_ratios': [30,1],
                                    'hspace': 0.1,
                                    'wspace': 0.05,
                                })
        # Axes for main heatmap and colorbar
        ax = axes[0]
        cbar_ax = axes[1]

    # Set to the size of A4 paper
    fig.set_size_inches(20.7, 20.27)

    # Set up default colormap if none was supplied
    if heatmap_cmap is None:
        heatmap_cmap = sns.cubehelix_palette(as_cmap=True)

    # Plot the heatmap
    sns.heatmap(data=-np.log10(pvalues_2d),
                ax=ax,
                linewidths=0.3,
                linecolor='lightblue',
                xticklabels=xlbls,
                yticklabels=ylbls,
                annot=counts_2d,
                annot_kws={
                    "size": ANNOTATION_FONT_SIZE,
                },
                fmt = 'g',
                vmin=min_pvalue,
                vmax=max_pvalue,
                cbar=True,
                cbar_ax=cbar_ax,
                cbar_kws={
                    "orientation": "vertical",
                    "pad":0.03,
                    "fraction":0.05,
                    "label":"-log(Pval)",
                },
                cmap=heatmap_cmap)

    # Annotate the sets of distances in the heatmap
    # NB the coordinates are in the 'data' coordinate system
    # see https://matplotlib.org/tutorials/advanced/transforms_tutorial.html
    midpoint = n_distances/2.0
    xpos = n_clusters/10.0
    for i,peak_file in enumerate(peaks):
        name = splitext(basename(peak_file))[0]
        ypos = i*n_distances
        ax.text(-xpos,ypos+midpoint,
                name,
                ha="center",
                va="center",
                rotation=90,
                size=PEAKSET_LABEL_FONT_SIZE)
        ax.plot([-0.65*xpos,
                 -0.75*xpos,
                 -0.75*xpos,
                 -0.65*xpos],
                [ypos+0.1,
                 ypos+0.1,
                 ypos+n_distances-0.1,
                 ypos+n_distances-0.1],
                linewidth=2,clip_on=False,color="gray")

    # Label the axes and move y-label over to accommodate extra
    # annotation on y-axis
    # NB the coordinates are in the 'axes' coordinate system
    # see https://matplotlib.org/tutorials/advanced/transforms_tutorial.html
    ax.set_ylabel(peaksets_axis_label,
                  fontsize=AXIS_LABEL_FONT_SIZE)
    ax.get_yaxis().set_label_coords(-0.13,0.5)

    # Set the fontsize for the tick labels
    for tick in ax.xaxis.get_major_ticks():
        tick.label1.set_fontsize(X_TICK_LABEL_FONT_SIZE)
        tick.label1.set_rotation('horizontal')
    for tick in ax.yaxis.get_major_ticks():
        tick.label1.set_fontsize(Y_TICK_LABEL_FONT_SIZE)
        tick.label1.set_rotation('horizontal')

    # Deal with the colorbar
    cbar_ax.yaxis.label.set_size(AXIS_LABEL_FONT_SIZE)
    # Set the tick label size
    # (see https://stackoverflow.com/a/53095480)
    cbar_ax.tick_params(labelsize=Y_TICK_LABEL_FONT_SIZE)

    # TADs heatmap
    if include_tads:
        sns.heatmap(data=-np.log10(tads_pvalues),
                    ax=tads_ax,
                    linewidths=0.3,
                    linecolor='lightblue',
                    xticklabels=xlbls,
                    yticklabels=[splitext(basename(f))[0]
                                 for f in peaks],
                    annot=tads_counts,
                    annot_kws={
                        "size": ANNOTATION_FONT_SIZE,
                    },
                    fmt = 'g',
                    vmin=min_pvalue,
                    vmax=max_pvalue,
                    cbar=True,
                    cbar_ax=tads_cbar_ax,
                    cbar_kws={
                        "orientation": "vertical",
                        "pad":0.03,
                        "fraction":0.05,
                        "label":"-log(Pval)",
                    },
                    cmap=heatmap_cmap)
        tads_ax.get_yaxis().set_label_coords(-0.9,0.5)

        # Set the fontsize for the tick labels
        for tick in tads_ax.xaxis.get_major_ticks():
            tick.label1.set_fontsize(X_TICK_LABEL_FONT_SIZE)
            tick.label1.set_rotation('horizontal')
        for tick in tads_ax.yaxis.get_major_ticks():
            tick.label1.set_fontsize(Y_TICK_LABEL_FONT_SIZE)
            tick.label1.set_rotation('horizontal')

        # Colorbar
        tads_cbar_ax.yaxis.label.set_size(AXIS_LABEL_FONT_SIZE)
        tads_cbar_ax.tick_params(labelsize=Y_TICK_LABEL_FONT_SIZE)

    # Set x axis legend
    if include_tads:
        ax = tads_ax
    ax.set_xlabel(clusters_axis_label,fontsize=AXIS_LABEL_FONT_SIZE)

    # Save to file
    fig.savefig(heatmap_file,format=heatmap_format)

def make_xlsx_file(xlsx_file,peaks,clusters,distances,pvalues,counts,
                   tads_pvalues=None,tads_counts=None):
    """
    Generate an XLSX file from enrichment data

    Arguments:
      xlsx_file (str): name/path for output XLSX file
      peaks (list): BED files containing the ChIP-seq peaks
      clusters (list): list of distances to calculate enrichments at
      distances (list): cluster files
      pvalues (numpy.array): Numpy array with pvalues from enrichment
        calculation
      counts (numpy.array): Numpy array with gene counts from enrichment
        calculation
      tads_pvalues (numpy.array): Numpy array with TADs pvalues from
        enrichment calculation (None if TADs not included)
      tads_counts (numpy.array): Numpy array with TADs counts from
        enrichment calculation (None if TADs not included)
    """
    # Convenience variables
    n_peaks = len(peaks)
    n_clusters = len(clusters)
    n_distances = len(distances)
    include_tads = (tads_pvalues is not None) and \
                   (tads_counts is not None)
    cluster_names = [os.path.splitext(os.path.basename(x))[0]
                     for x in clusters]

    # Output workbook
    xlsx_out = xlsxwriter.Workbook(xlsx_file)

    # Make separate sheets for each set of values
    ws_common_genes = xlsx_out.add_worksheet("Common Genes")
    ws_p_values = xlsx_out.add_worksheet("P values")

    # Set up formats
    fmt_center = xlsx_out.add_format({'align':'center'})

    # Get widths of peak set names
    width = 0
    for peaks_file in peaks:
        width = max(width,len(basename(peaks_file)))

    # Make header rows
    for ws in (ws_common_genes,ws_p_values):
        ws.write(1,0,"Peak set")
        ws.write(1,1,"Interval")
        for k,cluster in enumerate(clusters):
            ws.write(1,k+2,cluster_names[k])
        ws.merge_range(0,2,0,n_clusters+1,
                       "Clusters",
                       fmt_center)

    # Write the data
    for i,peaks_file in enumerate(peaks):
        for j,distance in enumerate(distances):
            row = i*n_distances + j + 2
            ws_common_genes.write(row,0,basename(peaks_file))
            ws_common_genes.write(row,1,distance)
            ws_p_values.write(row,0,basename(peaks_file))
            ws_p_values.write(row,1,distance)
            for k,cluster in enumerate(clusters):
                ws_common_genes.write(row,k+2,counts[i,j,k])
                ws_p_values.write(row,k+2,pvalues[i,j,k])

    # Set the column width for peak sets
    ws_common_genes.set_column(0,0,width*1.2)
    ws_p_values.set_column(0,0,width*1.2)

    # Output TADs data if specified
    if include_tads:
        ws_tads_common_genes = xlsx_out.add_worksheet("Common Genes (TADs)")
        ws_tads_p_values = xlsx_out.add_worksheet("P values (TADs)")
        # Make header rows
        for ws in (ws_tads_common_genes,ws_tads_p_values):
            ws.write(0,0,"Peak set")
            for k,cluster in enumerate(clusters):
                ws.write(1,k+1,cluster_names[k])
                ws.merge_range(0,1,0,n_clusters,
                               "Clusters",
                               fmt_center)
        # Write the data
        for i,peaks_file in enumerate(peaks):
            row = i+1
            ws_tads_common_genes.write(row,0,basename(peaks_file))
            ws_tads_p_values.write(row,0,basename(peaks_file))
            for k,cluster in enumerate(clusters):
                ws_tads_common_genes.write(row,k+1,tads_counts[i,k])
                ws_tads_p_values.write(row,k+1,tads_pvalues[i,k])
    # Set the column width for peak sets
    ws_common_genes.set_column(0,0,width*1.2)
    ws_p_values.set_column(0,0,width*1.2)
    if include_tads:
        ws_tads_common_genes.set_column(0,0,width*1.2)
        ws_tads_p_values.set_column(0,0,width*1.2)
    xlsx_out.close()

def write_raw_data(name,peaks,clusters,distances,pvalues,counts,
                   tads_pvalues=None,tads_counts=None,
                   output_directory=None):
    """
    Write the raw pvalue/count data to tab-delimited files

    Arguments:
      name (str): basename to use for output files
      peaks (list): BED files containing the ChIP-seq peaks
      clusters (list): list of distances to calculate enrichments at
      distances (list): cluster files
      pvalues (numpy.array): Numpy array with pvalues from enrichment
        calculation
      counts (numpy.array): Numpy array with gene counts from enrichment
        calculation
      tads_pvalues (numpy.array): Numpy array with TADs pvalues from
        enrichment calculation (None if TADs not included)
      tads_counts (numpy.array): Numpy array with TADs gene counts from
        enrichment calculation (None if TADs not included)
      output_directory (str): output directory to write files to (will
        be current working directory if not supplied)
    """
    # Convenience variables
    include_tads = (tads_pvalues is not None) and \
                   (tads_counts is not None)

    # Output directory
    if output_directory is None:
        output_directory = os.getcwd()
    output_directory = os.path.abspath(output_directory)

    # Dump pvalues and gene counts
    pval_filen = os.path.join(output_directory,'%s_pval.tsv' % name)
    count_filen = os.path.join(output_directory,'%s_count.tsv' % name)
    with io.open(pval_filen,'wt') as fpval:
        with io.open(count_filen,'wt') as fcount:
            for i,peaks_file in enumerate(peaks):
                for j,distance in enumerate(distances):
                    line_pval = [basename(peaks_file),distance]
                    line_count = [basename(peaks_file),distance]
                    for k,cluster in enumerate(clusters):
                        line_pval.append(pvalues[i,j,k])
                        line_count.append(int(counts[i,j,k]))
                    fpval.write("%s\n" % '\t'.join([str(x) for x in line_pval]))
                    fcount.write("%s\n" % '\t'.join([str(x) for x in line_count]))

    # Dump data for TADs
    if include_tads:
        tads_pval_filen = os.path.join(output_directory,
                                       '%s_tads_pval.tsv' % name)
        tads_count_filen = os.path.join(output_directory,
                                        '%s_tads_count.tsv' % name)
        with io.open(tads_pval_filen,'wt') as fpval:
            with io.open(tads_count_filen,'wt') as fcount:
                for i,peaks_file in enumerate(peaks):
                    line_pval = [basename(peaks_file)]
                    line_count = [basename(peaks_file)]
                    for k,cluster in enumerate(clusters):
                        line_pval.append(tads_pvalues[i,k])
                        line_count.append(int(tads_counts[i,k]))
                    fpval.write("%s\n" % '\t'.join([str(x) for x in line_pval]))
                    fcount.write("%s\n" % '\t'.join([str(x) for x in line_count]))



#!/usr/bin/env python
#
#     utils.py: utility functions for PEGS
#     Copyright (C) University of Manchester 2018-2020 Mudassar Iqbal, Peter Briggs
#

#######################################################################
# Imports
#######################################################################

import os
import io
from os import listdir
from os.path import isfile
from os.path import join
from os.path import abspath
from os.path import basename
from os.path import splitext
from fnmatch import fnmatch

#######################################################################
# Functions
#######################################################################

def find_exe(exe):
    """
    Locate an executable
    """
    if not os.path.isabs(exe):
        for p in os.environ['PATH'].split(os.pathsep):
            pth = join(p,exe)
            if isfile(pth) and os.access(pth,os.X_OK):
                return pth
    else:
        if os.path.isfile(exe) and os.access(exe,os.X_OK):
            return exe
    return None

def count_genes(bed_file):
    """
    Count the total number of genes in a BED file
    """
    print("Counting genes in %s" % basename(bed_file))
    m = 0
    with io.open(bed_file,'rt') as bed:
        for line in bed:
            m += 1
    print("Counted %d genes\n" % m)
    return m

def collect_files(d):
    """
    Collect files from a directory
    """
    return sort_files([abspath(join(d,f))
                       for f in listdir(d) if isfile(join(d,f))
                       and not f.startswith('.')
                       and not f.endswith('~')])

def sort_files(f):
    """
    Sort files based on integer components
    """
    return sorted(f,key=split_file_name_for_sort)

def split_file_name_for_sort(f):
    """
    Returns tuple of string and integer components for sorting

    For example:

    "cluster_10.txt" -> ("cluster_",10,".txt")
    """
    components = []
    current_component = ''
    component_is_digits = False
    # Loop over characters in the input string
    for c in str(f):
        if current_component:
            # Already building a group of characters
            if (c.isdigit() and not component_is_digits) or \
               (not c.isdigit() and component_is_digits):
                # Next character is a different type from
                # the current group, so the current group
                # is complete
                if component_is_digits:
                    # Convert digits to integer
                    current_component = int(current_component)
                components.append(current_component)
                # Reset for a new group of characters
                current_component = ''
                component_is_digits = False
            else:
                # Next character is same type as current
                # group, just append it
                current_component += c
        if not current_component:
            # Next character starts a new group
            current_component += c
            component_is_digits = c.isdigit()
    # Finished looping, check if there is a final
    # unprocessed character group
    if current_component:
        if component_is_digits:
            # Convert digits to integer
            current_component = int(current_component)
        components.append(current_component)
    # Return the tuple of elements
    return tuple(components)

def intersection_file_basename(interval_file,peak_file,distance=None):
    """
    Generate a name for an intersection file
    """
    return "%s.%s%s" % (splitext(basename(interval_file))[0],
                        splitext(basename(peak_file))[0],
                        (".%s" % distance if distance is not None else ""))



#!/usr/bin/env python
#
#     cli.py: CLI for gene cluster enrichment analysis
#     Copyright (C) University of Manchester 2018-2021 Mudassar Iqbal, Peter Briggs
#
from builtins import str
import os
import argparse
import logging
# Deal with matplotlib backend before importing seaborn
# See https://stackoverflow.com/a/50089385/579925
import matplotlib
if os.environ.get('DISPLAY','') == '':
   print('No display found: using non-interactive Agg backend')
   matplotlib.use('Agg')
import seaborn as sns
import pathlib2
from .pegs import pegs_main
from .intervals import make_gene_interval_file
from .bedtools import fetch_bedtools
from .bedtools import bedtools_version
from .utils import find_exe
from .utils import collect_files
from .utils import sort_files
from . import get_version

# Description
PEGS_DESCRIPTION = "PEGS: Peak-set Enrichment of Gene-Sets"

# Citation
PEGS_CITATION = """
If you use PEGS in your research then please cite:

* Briggs P, Hunter AL, Yang Sh et al.
  PEGS: An efficient tool for gene set enrichment within defined
  sets of genomic intervals [version 1; peer review: 2 approved].
  F1000Research 2021, 10:570
  (https://doi.org/10.12688/f1000research.53926.1)
"""

# Default set of distances for enrichment calculation
DEFAULT_DISTANCES = [5000,25000,50000,100000,150000,200000]

# Built in gene interval files
BUILTIN_GENE_INTERVALS = {
    "hg38": "refGene_hg38_120719_intervals.bed",
    "mm10": "refGene_mm10_120719_intervals.bed",
}

# Types for cubehelix_palette options
CUBEHELIX_PALETTE_TYPES = {
    'n_colors': int,
    'start': float,
    'rot': float,
    'gamma': float,
    'hue': float,
    'dark': float,
    'light': float,
    'reverse': bool,
}

def pegs():
    # Create command line parser
    p = argparse.ArgumentParser(description=PEGS_DESCRIPTION)
    p.add_argument("gene_intervals",
                   metavar="GENE_INTERVALS",
                   help="either name of a built-in set of gene "
                   "intervals (%s), or a BED file with gene interval "
                   "data" %
                   ','.join(["'%s'" % x for x in BUILTIN_GENE_INTERVALS]))
    p.add_argument('--version',action='version',version=get_version())
    p.add_argument("-p","--peaks",
                   metavar="PEAK_SET_FILE",
                   dest="peaks",
                   action="store",
                   required=True,
                   nargs="+",
                   help="one or more input peak set files (BED format)")
    p.add_argument("-g","--genes",
                   metavar="GENE_CLUSTER_FILE",
                   dest="clusters",
                   action="store",
                   required=True,
                   nargs="+",
                   help="one or more input gene cluster files (one gene "
                   "per line)")
    p.add_argument("-t","--tads",metavar="TADS_FILE",
                   dest="tads_file",
                   action="store",
                   help="BED file with topologically associating "
                   "domains (TADs)")
    p.add_argument("-d","--distances",
                   metavar="DISTANCE",
                   dest="distances",
                   action="store",
                   nargs="+",
                   help="specify distance(s) to calculate enrichments "
                   "for (if no distances are specified then the default "
                   "set will be used i.e. %s)" %
                   ' '.join([str(x) for x in DEFAULT_DISTANCES]))
    output_options = p.add_argument_group("Output options")
    output_options.add_argument("--name",metavar="BASENAME",
                                dest="name",
                                action='store',
                                default="pegs",
                                help="basename for output files (default: "
                                "'pegs')")
    output_options.add_argument("-o",metavar="OUTPUT_DIRECTORY",
                                dest="output_directory",
                                action="store",
                                default=None,
                                help="specify directory to write output "
                                "files to (default: write to current "
                                "directory)")
    output_options.add_argument("-m",metavar="HEATMAP",
                                dest="output_heatmap",
                                action="store",
                                default=None,
                                help="destination for output heatmap; "
                                "image format is implicitly determined by "
                                "the file extension (e.g. '.png','.svg' "
                                "etc) unless overridden by the --format "
                                "option (default: 'BASENAME_heatmap.FORMAT')")
    output_options.add_argument("-x",metavar="XLSX",
                                dest="output_xlsx",
                                action="store",
                                default=None,
                                help="destination for output XLSX file "
                                "with the raw enrichment data (default: "
                                "'BASENAME_results.xlsx')")
    heatmap_options = p.add_argument_group("Heatmap options")
    heatmap_options.add_argument("--format",
                                 dest="heatmap_format",
                                 metavar="FORMAT",
                                 action="store",
                                 default = None,
                                 help="explicitly specify the image format "
                                 "for the output heatmap; note that if this "
                                 "option is specified then it will override "
                                 "the format implied by the specified with "
                                 "the -m option (default: 'png')")
    heatmap_options.add_argument("--x-label",
                                 metavar="CLUSTERS_AXIS_LABEL",
                                 dest="clusters_axis_label",
                                 action="store",
                                 default=None,
                                 help="set a custom label for the X "
                                 "(clusters) axis")
    heatmap_options.add_argument("--y-label",
                                 metavar="PEAKSETS_AXIS_LABEL",
                                 dest="peaksets_axis_label",
                                 action="store",
                                 default=None,
                                 help="set a custom label for the Y "
                                 "(peak sets) axis")
    g = heatmap_options.add_mutually_exclusive_group()
    g.add_argument("--color",
                   dest="heatmap_color",
                   metavar="COLOR",
                   action="store",
                   default=None,
                   help="specify a base color to use for the heatmap "
                   "(NB not compatible with --heatmap-palette)")
    g.add_argument("--heatmap-palette",
                   dest="heatmap_palette_options",
                   metavar="OPTION=VALUE",
                   action="store",
                   nargs="+",
                   default = None,
                   help="advanced option to specify custom palette "
                   "settings for the output heatmap (e.g. 'start=0.5', "
                   "'rot=0' etc). Available options are those listed in "
                   "the 'cubehelix_palette' documentation at "
                   "https://seaborn.pydata.org/generated/"
                   "seaborn.cubehelix_palette.html (NB not compatible "
                   "with --color)")
    advanced_options = p.add_argument_group("Advanced options")
    advanced_options.add_argument("-k","--keep-intersection-files",
                                  dest="keep_intersection_files",
                                  action="store_true",
                                  help="keep the intermediate intersection "
                                  "files (useful for debugging)")
    advanced_options.add_argument("--dump-raw-data",
                                  dest="dump_raw_data",
                                  action="store_true",
                                  help="dump the raw data (gene counts and "
                                  "p-values) to TSV files (for debugging)")
    args = p.parse_args()
    # Deal with peak and cluster files
    peaks = sort_files(args.peaks)
    clusters = sort_files(args.clusters)
    # Generate list of distances
    if not args.distances:
        # Defaults
        distances = [d for d in DEFAULT_DISTANCES]
    else:
        # Assemble from command line
        distances = list()
        for d in args.distances:
            for x in d.split(','):
                distances.append(int(x))
    distances = sorted(distances)
    # Check if using built-in interval data
    gene_interval_file = args.gene_intervals
    try:
        gene_interval_file = BUILTIN_GENE_INTERVALS[gene_interval_file]
        p = os.path.dirname(__file__)
        while p != os.sep:
            f = os.path.join(p,"pegs-%s" % get_version(),gene_interval_file)
            if os.path.exists(f):
                gene_interval_file = f
                break
            else:
                p = os.path.dirname(p)
    except KeyError:
        # Not found, ignore
        pass
    # Check TADs file is actually a file
    if args.tads_file:
       if not os.path.exists(args.tads_file):
          logging.fatal("TADs file '%s' doesn't exist" % args.tads_file)
          return 1
       elif os.path.isdir(args.tads_file):
          logging.fatal("TADs file '%s' is a directory (must be a file)"
                        % args.tads_file)
          return 1
    # Build colormap for heatmap
    heatmap_cmap = None
    if args.heatmap_color:
        # Construct non-default colormap using the
        # seaborn lightpalette function
        heatmap_cmap = sns.light_palette(color=args.heatmap_color,
                                         as_cmap=True)
    elif args.heatmap_palette_options is not None:
        # Construct non-default colormap using the
        # options supplied by the user
        heatmap_palette_options = {
            'n_colors': 6,
            'start': 0,
            'rot': 0.4,
            'gamma': 1.0,
            'hue': 0.8,
            'light': 0.85,
            'dark': 0.15,
            'reverse': False,
        }
        for o in args.heatmap_palette_options:
            key,value = o.split("=")
            if key not in heatmap_palette_options:
                logging.warning("Unrecognised palette option: '%s'"
                                % key)
            else:
                heatmap_palette_options[key] = \
                        CUBEHELIX_PALETTE_TYPES[key](value)
        heatmap_cmap = sns.cubehelix_palette(as_cmap=True,
                                             **heatmap_palette_options)
    # Report version and authors etc
    print("%s %s" % (PEGS_DESCRIPTION,get_version()))
    print("""
Efficiently calculate enrichments of gene clusters in
multiple genomic intervals data (e.g. ChIP-seq peak-sets)
at different distances

Copyright University of Manchester
Faculty of Biology Medicine and Health
Authors: Mudassar Iqbal, Peter Briggs
""")
    print(PEGS_CITATION)

    print("====PEGS is starting====")

    # Add PEGS 'bin' directory in user's home area to PATH
    # NB this might not exist
    pegs_dir = os.path.join(str(pathlib2.Path.home()),".pegs")
    pegs_bin_dir = os.path.join(pegs_dir,"bin")
    os.environ['PATH'] = "%s%s%s" % (os.environ['PATH'],
                                     os.pathsep,
                                     pegs_bin_dir)

    # Locate bedtools executable
    bedtools_exe = find_exe("bedtools")
    if not bedtools_exe:
        # Not found
        logging.warning("'bedtools' not found")
        # Attempt to get bedtools
        bedtools_exe = fetch_bedtools(install_dir=pegs_bin_dir,
                                      create_install_dir=True)
        if not bedtools_exe:
            logging.fatal("Failed to fetch 'bedtools'")
            return 1
    print("Found %s (%s)\n" % (bedtools_version(bedtools_exe),
                               bedtools_exe))

    # Calculate the enrichments
    pegs_main(genes_file=gene_interval_file,
              distances=distances,
              peaks=peaks,
              clusters=clusters,
              tads_file=args.tads_file,
              name=args.name,
              heatmap=args.output_heatmap,
              xlsx=args.output_xlsx,
              output_directory=args.output_directory,
              keep_intersection_files=
              args.keep_intersection_files,
              clusters_axis_label=args.clusters_axis_label,
              peaksets_axis_label=args.peaksets_axis_label,
              heatmap_cmap=heatmap_cmap,
              heatmap_format=args.heatmap_format,
              dump_raw_data=args.dump_raw_data)

def mk_pegs_intervals():
    # Create command line parser
    p = argparse.ArgumentParser()
    p.add_argument("refgene_file",
                   metavar="REFGENE_FILE",
                   help="refGene annotation data for the genome "
                   "of interest")
    p.add_argument("gene_interval_file",
                   metavar="GENE_INTERVAL_FILE",
                   nargs='?',
                   help="destination for output BED file with "
                   "gene interval data (default: "
                   "'<REFGENE_FILE>_intervals.bed')")
    p.add_argument('--version',action='version',version=get_version())
    args = p.parse_args()
    # Report version
    print("MK_PEGS_INTERVALS %s\n" % get_version())
    # Generate the gene interval file
    make_gene_interval_file(args.refgene_file,
                            args.gene_interval_file)
